import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.model.server_config import ServerConfig
from bot_data.model.team_member_type_enum import TeamMemberTypeEnum
from bot_data.model.technician_config import TechnicianConfig
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class PermissionService(PermissionServiceABC):
    def __init__(
        self,
        technician_settings: TechnicianConfig,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        config: ConfigurationABC,
    ):
        PermissionServiceABC.__init__(self)
        self._logger = logger
        self._bot = bot
        self._config = config
        self._technician_settings = technician_settings

        self._admin_role_ids: dict[int, list[int]] = {}
        self._admin_roles: dict[int, list[discord.Role]] = {}
        self._admins: dict[int, list[discord.Member]] = {}

        self._moderator_role_ids: dict[int, list[int]] = {}
        self._moderator_roles: dict[int, list[discord.Role]] = {}
        self._moderators: dict[int, list[discord.Member]] = {}

        self._technician_ids: list[int] = technician_settings.technician_ids.to_list()
        self._technicians: list[discord.Member] = []

    def on_ready(self):
        for guild in self._bot.guilds:
            guild: discord.Guild = guild
            self._logger.debug(__name__, f"Validate permission settings")

            for technician_id in self._technician_ids:
                technician = guild.get_member(technician_id)
                if technician is None:
                    continue
                self._technicians.append(technician)

            settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{guild.id}")
            if settings is None:
                self._logger.error(__name__, "Server settings not found")
                return

            self._admin_role_ids[guild.id] = (
                settings.team_role_ids.where(lambda x: x.team_member_type == TeamMemberTypeEnum.admin)
                .select(lambda x: x.role_id)
                .to_list()
            )
            self._moderator_role_ids[guild.id] = (
                settings.team_role_ids.where(lambda x: x.team_member_type == TeamMemberTypeEnum.moderator)
                .select(lambda x: x.role_id)
                .to_list()
            )

            admin_roles = []
            admins = []

            mod_roles = []
            mods = []

            for role in guild.roles:
                role: discord.Role = role

                if role.id in self._admin_role_ids[guild.id]:
                    admin_roles.append(role)
                    self._logger.trace(__name__, f"Added admin role {role}")

                    for member in role.members:
                        admins.append(member)
                        self._logger.trace(__name__, f"Added admin {member}")

                if role.id in self._moderator_role_ids[guild.id]:
                    mod_roles.append(role)
                    self._logger.trace(__name__, f"Added moderator role {role}")

                    for member in role.members:
                        mods.append(member)
                        self._logger.trace(__name__, f"Added moderator {member}")

            self._admin_roles[guild.id] = admin_roles
            self._admins[guild.id] = admins
            self._moderator_roles[guild.id] = mod_roles
            self._moderators[guild.id] = mods

    def on_member_update(self, before: discord.Member, after: discord.Member):
        g_id = after.guild.id

        for admin_role in self._admin_roles[g_id]:
            if admin_role in before.roles and admin_role not in after.roles:
                self._admins[g_id].remove(after)
                self._logger.trace(__name__, f"Removed {after.id} from admins")

            elif admin_role in after.roles and admin_role not in before.roles:
                self._admins[g_id].append(after)
                self._logger.trace(__name__, f"Added {after.id} to admins")

        for moderator_role in self._moderator_roles[g_id]:
            if moderator_role in before.roles and moderator_role not in after.roles:
                self._moderators[g_id].remove(after)
                self._logger.trace(__name__, f"Removed {after.id} from moderators")

            elif moderator_role in after.roles and moderator_role not in before.roles:
                self._moderators[g_id].append(after)
                self._logger.trace(__name__, f"Added {after.id} to moderators")

    def get_admin_role_ids(self, g_id: int) -> list[int]:
        return self._admin_role_ids[g_id]

    def get_admin_roles(self, g_id: int) -> list[discord.Role]:
        return self._admin_roles[g_id]

    def get_admins(self, g_id: int) -> list[discord.Member]:
        return self._admins[g_id]

    def get_moderator_role_ids(self, g_id: int) -> list[int]:
        return self._moderator_role_ids[g_id]

    def get_moderator_roles(self, g_id: int) -> list[discord.Role]:
        return self._moderator_roles[g_id]

    def get_moderators(self, g_id: int) -> list[discord.Member]:
        return self._moderators[g_id]

    def get_technicians(self) -> list[discord.Member]:
        return self._technicians

    def is_member_admin(self, member: discord.Member) -> bool:
        return member is not None and member.guild.id in self._admins and member in self._admins[member.guild.id]

    def is_member_moderator(self, member: discord.Member) -> bool:
        return (
            member is not None
            and member.guild.id in self._moderators
            and member in self._moderators[member.guild.id]
            or self.is_member_admin(member)
        )

    def is_member_technician(self, member: discord.Member) -> bool:
        return member is not None and member in self._technicians
